@@ -126,7 +126,8 @@ class MPState(object):
 
               MPSetting('basealt', int, 0, 'Base Altitude', range=(0,30000), increment=1, tab='Altitude'),
               MPSetting('wpalt', int, 100, 'Default WP Altitude', range=(0,10000), increment=1),
-              MPSetting('rallyalt', int, 90, 'Default Rally Altitude', range=(0,10000), increment=1)]
+              MPSetting('rallyalt', int, 90, 'Default Rally Altitude', range=(0,10000), increment=1),
+              MPSetting('terrainalt', bool, False, 'Use terrain altitudes')]
             )
 
         self.completions = {
