@@ -19,7 +19,7 @@ class MPMenuGeneric(object):
     def _append(self, menu):
         '''append this menu item to a menu'''
         pass
-    
+
     def __str__(self):
         return "MPMenuGeneric()"
 
@@ -41,18 +41,28 @@ class MPMenuSeparator(MPMenuGeneric):
 
 class MPMenuItem(MPMenuGeneric):
     '''a MP menu item'''
-    def __init__(self, name, description='', returnkey=None):
+    def __init__(self, name, description='', returnkey=None, handler=None):
         MPMenuGeneric.__init__(self)
         self.name = name
         self.description = description
         self.returnkey = returnkey
-        
+        self.handler = handler
+        self.handler_result = None
+
     def find_selected(self, event):
         '''find the selected menu item'''
         if event.GetId() == self.id():
             return self
         return None
 
+    def call_handler(self):
+        '''optionally call a handler function'''
+        if self.handler is None:
+            return
+        call = getattr(self.handler, 'call', None)
+        if call is not None:
+            self.handler_result = call()
+
     def id(self):
         '''id used to identify the returned menu items
         uses a 16 bit unsigned integer'''
@@ -69,10 +79,10 @@ class MPMenuItem(MPMenuGeneric):
 
 class MPMenuCheckbox(MPMenuItem):
     '''a MP menu item as a checkbox'''
-    def __init__(self, name, description='', returnkey=None, checked=False):
-        MPMenuItem.__init__(self, name, description=description, returnkey=returnkey)
+    def __init__(self, name, description='', returnkey=None, checked=False, handler=None):
+        MPMenuItem.__init__(self, name, description=description, returnkey=returnkey, handler=handler)
         self.checked = checked
-        
+
     def find_selected(self, event):
         '''find the selected menu item'''
         if event.GetId() == self.id():
@@ -94,8 +104,8 @@ class MPMenuCheckbox(MPMenuItem):
 
 class MPMenuRadio(MPMenuItem):
     '''a MP menu item as a radio item'''
-    def __init__(self, name, description='', returnkey=None, selected=None, items=[]):
-        MPMenuItem.__init__(self, name, description=description, returnkey=returnkey)
+    def __init__(self, name, description='', returnkey=None, selected=None, items=[], handler=None):
+        MPMenuItem.__init__(self, name, description=description, returnkey=returnkey, handler=handler)
         self.items = items
         self.choice = 0
         self.initial = selected
@@ -126,7 +136,7 @@ class MPMenuRadio(MPMenuItem):
             if self.items[i] == self.initial:
                 submenu.Check(self.id()+i, True)
         menu.AppendMenu(-1, self.name, submenu)
-        
+
     def __str__(self):
         return "MPMenuRadio(%s,%s,%s,%s)" % (self.name, self.description, self.returnkey, self.get_choice())
 
@@ -137,13 +147,17 @@ class MPMenuSubMenu(MPMenuGeneric):
         MPMenuGeneric.__init__(self)
         self.name = name
         self.items = items
-        
+
     def add(self, items, addto=None):
         '''add more items to a sub-menu'''
         if not isinstance(items, list):
             items = [items]
         self.items.extend(items)
 
+    def combine(self, submenu):
+        '''combine a new menu with an existing one'''
+        self.items.extend(submenu.items)
+
     def wx_menu(self):
         '''return a wx.Menu() for this menu'''
         menu = wx.Menu()
@@ -196,10 +210,35 @@ class MPMenuTop(object):
                 return ret
         return None
 
+class MPMenuCallFileDialog(object):
+    '''used to create a file dialog callback'''
+    def __init__(self, flags=wx.FD_OPEN, title='Filename', wildcard='*.*'):
+        self.flags = flags
+        self.title = title
+        self.wildcard = wildcard
 
+    def call(self):
+        '''show a file dialog'''
+        dlg = wx.FileDialog(None, self.title, '', "", self.wildcard, self.flags)
+        if dlg.ShowModal() != wx.ID_OK:
+            return None
+        return dlg.GetPath()
+
+class MPMenuCallTextDialog(object):
+    '''used to create a value dialog callback'''
+    def __init__(self, title='Enter Value', default=''):
+        self.title = title
+        self.default = default
+
+    def call(self):
+        '''show a value dialog'''
+        dlg = wx.TextEntryDialog(None, self.title, self.title, defaultValue=str(self.default))
+        if dlg.ShowModal() != wx.ID_OK:
+            return None
+        return dlg.GetValue()
 
 if __name__ == '__main__':
-    from MAVProxy.modules.mavproxy_map.mp_image import MPImage
+    from MAVProxy.modules.lib.mp_image import MPImage
     import time
     im = MPImage(mouse_events=True,
                  key_events=True,
@@ -223,7 +262,7 @@ if __name__ == '__main__':
                                                        items=['Red','Green','Blue']),
                                            MPMenuRadio('Shapes',
                                                        items=['Circle','Square','Triangle'])])])
-    
+
     im.set_menu(menu)
 
     popup = MPMenuSubMenu('A Popup',
@@ -232,7 +271,7 @@ if __name__ == '__main__':
                                  MPMenuItem('Sub3')])
 
     im.set_popup_menu(popup)
-    
+
     while im.is_alive():
         for event in im.events():
             if isinstance(event, MPMenuItem):
@@ -241,4 +280,3 @@ if __name__ == '__main__':
             else:
                 print(event)
         time.sleep(0.1)
-        
