@@ -5,6 +5,7 @@ import os, time
 from pymavlink import mavwp, mavutil
 from MAVProxy.modules.lib import mp_util
 from MAVProxy.modules.lib import mp_module
+from MAVProxy.modules.lib.mp_menu import *
 
 class FenceModule(mp_module.MPModule):
     def __init__(self, mpstate):
@@ -17,15 +18,42 @@ class FenceModule(mp_module.MPModule):
         self.healthy = True
         self.add_command('fence', self.cmd_fence,
                          "geo-fence management",
-                         ["<draw|list|clear|enable|disable>",
+                         ["<draw|list|clear|enable|disable|move|remove>",
                           "<load|save> (FILENAME)"])
 
+        self.have_list = False
+
         if self.continue_mode and self.logdir != None:
             fencetxt = os.path.join(self.logdir, 'fence.txt')
             if os.path.exists(fencetxt):
                 self.fenceloader.load(fencetxt)
+                self.have_list = True
                 print("Loaded fence from %s" % fencetxt)
 
+        self.menu_added_console = False
+        self.menu_added_map = False
+        self.menu = MPMenuSubMenu('Fence',
+                                  items=[MPMenuItem('Clear', 'Clear', '# fence clear'),
+                                         MPMenuItem('List', 'List', '# fence list'),
+                                         MPMenuItem('Load', 'Load', '# fence load ',
+                                                    handler=MPMenuCallFileDialog(flags=wx.FD_OPEN,
+                                                                                 title='Fence Load',
+                                                                                 wildcard='*.fen')),
+                                         MPMenuItem('Save', 'Save', '# fence save ',
+                                                    handler=MPMenuCallFileDialog(flags=wx.FD_SAVE|wx.FD_OVERWRITE_PROMPT,
+                                                                                 title='Fence Save',
+                                                                                 wildcard='*.fen')),
+                                         MPMenuItem('Draw', 'Draw', '# fence draw')])
+
+    def idle_task(self):
+        '''called on idle'''
+        if not self.menu_added_console and self.module('console') is not None:
+            self.menu_added_console = True
+            self.module('console').add_menu(self.menu)
+        if not self.menu_added_map and self.module('map') is not None:
+            self.menu_added_map = True
+            self.module('map').add_menu(self.menu)
+
     def mavlink_packet(self, m):
         '''handle and incoming mavlink packet'''
         if m.get_type() == "FENCE_STATUS":
@@ -40,21 +68,21 @@ class FenceModule(mp_module.MPModule):
             elif self.present == True and present == False:
                 self.say("fence removed")
             self.present = present
-        
+
             enabled = ((m.onboard_control_sensors_enabled & bits) == bits)
             if self.enabled == False and enabled == True:
                 self.say("fence enabled")
             elif self.enabled == True and enabled == False:
                 self.say("fence disabled")
             self.enabled = enabled
-            
+
             healthy = ((m.onboard_control_sensors_health & bits) == bits)
             if self.healthy == False and healthy == True:
                 self.say("fence OK")
             elif self.healthy == True and healthy == False:
                 self.say("fence breach")
-            self.healthy = healthy 
-    
+            self.healthy = healthy
+
             #console output for fence:
             if self.enabled == False:
                 self.console.set_status('Fence', 'FEN', row=0, fg='grey')
@@ -62,7 +90,7 @@ class FenceModule(mp_module.MPModule):
                 self.console.set_status('Fence', 'FEN', row=0, fg='green')
             elif self.enabled == True and self.healthy == False:
                 self.console.set_status('Fence', 'FEN', row=0, fg='red')
-    
+
     def set_fence_enabled(self, do_enable):
         '''Enable or disable fence'''
         self.master.mav.command_long_send(
@@ -70,13 +98,62 @@ class FenceModule(mp_module.MPModule):
             self.target_component,
             mavutil.mavlink.MAV_CMD_DO_FENCE_ENABLE, 0,
             do_enable, 0, 0, 0, 0, 0, 0)
-    
+
+    def cmd_fence_move(self, args):
+        '''handle fencepoint move'''
+        if len(args) < 1:
+            print("Usage: fence move FENCEPOINTNUM")
+            return
+        if not self.have_list:
+            print("Please list fence points first")
+            return
+
+        idx = int(args[0])
+        if idx <= 0 or idx > self.fenceloader.count():
+            print("Invalid fence point number %u" % idx)
+            return
+
+        try:
+            latlon = self.module('map').click_position
+        except Exception:
+            print("No map available")
+            return
+        if latlon is None:
+            print("No map click position available")
+            return
+
+        # note we don't subtract 1, as first fence point is the return point
+        self.fenceloader.move(idx, latlon[0], latlon[1])
+        if self.send_fence():
+            print("Moved fence point %u" % idx)
+
+    def cmd_fence_remove(self, args):
+        '''handle fencepoint remove'''
+        if len(args) < 1:
+            print("Usage: fence remove FENCEPOINTNUM")
+            return
+        if not self.have_list:
+            print("Please list fence points first")
+            return
+
+        idx = int(args[0])
+        if idx <= 0 or idx > self.fenceloader.count():
+            print("Invalid fence point number %u" % idx)
+            return
+
+        # note we don't subtract 1, as first fence point is the return point
+        self.fenceloader.remove(idx)
+        if self.send_fence():
+            print("Removed fence point %u" % idx)
+        else:
+            print("Failed to remove fence point %u" % idx)
+
     def cmd_fence(self, args):
         '''fence commands'''
         if len(args) < 1:
             self.print_usage()
             return
-    
+
         if args[0] == "enable":
             self.set_fence_enabled(1)
         elif args[0] == "disable":
@@ -88,6 +165,10 @@ class FenceModule(mp_module.MPModule):
             self.load_fence(args[1])
         elif args[0] == "list":
             self.list_fence(None)
+        elif args[0] == "move":
+            self.cmd_fence_move(args[1:])
+        elif args[0] == "remove":
+            self.cmd_fence_remove(args[1:])
         elif args[0] == "save":
             if len(args) != 2:
                 print("usage: fence save <filename>")
@@ -98,32 +179,36 @@ class FenceModule(mp_module.MPModule):
                 print("usage: fence show <filename>")
                 return
             self.fenceloader.load(args[1])
+            self.have_list = True
         elif args[0] == "draw":
             if not 'draw_lines' in self.mpstate.map_functions:
                 print("No map drawing available")
-                return        
+                return
             self.mpstate.map_functions['draw_lines'](self.fence_draw_callback)
             print("Drawing fence on map")
         elif args[0] == "clear":
             self.param_set('FENCE_TOTAL', 0, 3)
         else:
             self.print_usage()
-    
+
     def load_fence(self, filename):
         '''load fence points from a file'''
         try:
             self.fenceloader.target_system = self.target_system
             self.fenceloader.target_component = self.target_component
             self.fenceloader.load(filename)
-        except Exception, msg:
+        except Exception as msg:
             print("Unable to load %s - %s" % (filename, msg))
             return
         print("Loaded %u geo-fence points from %s" % (self.fenceloader.count(), filename))
         self.send_fence()
-    
+
     def send_fence(self):
         '''send fence points from fenceloader'''
         # must disable geo-fencing when loading
+        self.fenceloader.target_system = self.target_system
+        self.fenceloader.target_component = self.target_component
+        self.fenceloader.reindex()
         action = self.get_mav_param('FENCE_ACTION', mavutil.mavlink.FENCE_ACTION_NONE)
         self.param_set('FENCE_ACTION', mavutil.mavlink.FENCE_ACTION_NONE, 3)
         self.param_set('FENCE_TOTAL', self.fenceloader.count(), 3)
@@ -133,15 +218,16 @@ class FenceModule(mp_module.MPModule):
             p2 = self.fetch_fence_point(i)
             if p2 is None:
                 self.param_set('FENCE_ACTION', action, 3)
-                return
+                return False
             if (p.idx != p2.idx or
                 abs(p.lat - p2.lat) >= 0.00003 or
                 abs(p.lng - p2.lng) >= 0.00003):
                 print("Failed to send fence point %u" % i)
                 self.param_set('FENCE_ACTION', action, 3)
-                return
+                return False
         self.param_set('FENCE_ACTION', action, 3)
-    
+        return True
+
     def fetch_fence_point(self ,i):
         '''fetch one fence point'''
         self.master.mav.fence_fetch_point_send(self.target_system,
@@ -158,7 +244,7 @@ class FenceModule(mp_module.MPModule):
             self.console.error("Failed to fetch point %u" % i)
             return None
         return p
-    
+
     def fence_draw_callback(self, points):
         '''callback from drawing a fence'''
         self.fenceloader.clear()
@@ -175,7 +261,8 @@ class FenceModule(mp_module.MPModule):
         # close it
         self.fenceloader.add_latlon(points[0][0], points[0][1])
         self.send_fence()
-    
+        self.have_list = True
+
     def list_fence(self, filename):
         '''list fence points, optionally saving to a file'''
         self.fenceloader.clear()
@@ -188,11 +275,11 @@ class FenceModule(mp_module.MPModule):
             if p is None:
                 return
             self.fenceloader.add(p)
-    
+
         if filename is not None:
             try:
                 self.fenceloader.save(filename)
-            except Exception, msg:
+            except Exception as msg:
                 print("Unable to save %s - %s" % (filename, msg))
                 return
             print("Saved %u geo-fence points to %s" % (self.fenceloader.count(), filename))
@@ -204,9 +291,10 @@ class FenceModule(mp_module.MPModule):
             fencetxt = os.path.join(self.status.logdir, 'fence.txt')
             self.fenceloader.save(fencetxt)
             print("Saved fence to %s" % fencetxt)
-    
+        self.have_list = True
+
     def print_usage(self):
-        print("usage: fence <enable|disable|list|load|save|clear|draw>")
+        print("usage: fence <enable|disable|list|load|save|clear|draw|move|remove>")
 
 def init(mpstate):
     '''initialise module'''
